"""Unit tests for app callbacks."""
import pytest
from plotly.graph_objs import Figure

from nerv.callbacks import plot_scatter_func
from nerv.utility import extract_metadata


@pytest.mark.parametrize(
    "clickData, metadata, x, y",
    [
        ("histogram", "histogram", "prevent-AD-FSL", None),
        ("histogram", "histogram", None, "prevent-AD-FreeSurfer"),
        ("scatter", "scatter", "prevent-AD-FSL", "prevent-AD-FreeSurfer"),
    ],
    indirect=["clickData", "metadata"],
)
def test_extract_metadata(metadata, clickData, x, y, df):
    """
    Tests whether extract_metadata condition that determines from which graph the
    clickData is coming work as intended.
    It utilizes clickData, metadata, and df fixtures for testing.
    """
    assert metadata == extract_metadata(clickData, x, y, df)


@pytest.mark.parametrize(
    "x, y, exp_x, exp_y",
    [
        (None, "prevent-AD-FreeSurfer", "prevent-AD-FSL", "compass-nd-IZK"),
        ("compass-nd-CSM", None, "prevent-AD-FSL", "compass-nd-IZK"),
        (None, None, "prevent-AD-FSL", "compass-nd-IZK"),
        (
            "prevent-AD-FreeSurfer",
            "compass-nd-CSM",
            "prevent-AD-FreeSurfer",
            "compass-nd-CSM",
        ),
    ],
)
def test_plot_scatter_func(x, y, exp_x, exp_y, df):
    """
    Tests whether plot_scatter_func returns an instance of Figure class and
    whether the plot axes are updated properly based on the input.
    It utilizes df fixtures for testing.
    """
    output = plot_scatter_func(x, y, df)
    assert isinstance(output, Figure)
    assert output["layout"]["xaxis"]["title"]["text"] == exp_x
    assert output["layout"]["yaxis"]["title"]["text"] == exp_y



"""
Test fixtures.
"""
import os

import pytest

from nerv.utility import process_files


@pytest.fixture(scope="session")
def path():
    """
    Provides the path of directory containing test data.

    Returns
    -------
    str
        Path of directory containing test data.
    """
    test_dir = os.path.dirname(os.path.abspath(__file__))
    print("Using", os.path.join(test_dir, "data"), "as the path for app")
    yield os.path.join(test_dir, "data")


@pytest.fixture(scope="session")
def df(path):
    """
    Provides a dataframe containing test data.
    It utilizes process_files function to generate
    the dataframe.

    Returns
    -------
    pandas.core.frame.DataFrame
        A dataframe containing test data.
    """

    yield process_files(path)


@pytest.fixture()
def clickData(request):
    if request.param == "histogram":
        return {
            "points": [
                {
                    "curveNumber": 7,
                    "pointNumber": 491,
                    "pointIndex": 491,
                    "x": 3489,
                    "y": "some_dataset-pipeline",
                    "bbox": {
                        "x0": 484.66,
                        "x1": 490.66,
                        "y0": 127.27,
                        "y1": 133.26999999999998,
                    },
                    "customdata": [
                        "some_subject",
                        "some_dataset-pipeline",
                        {
                            "first_step": {
                                "inputID": 6435319,
                                "toolConfigID": 678,
                                "taskID": 5616972,
                                "status": "Completed",
                                "outputID": 4283122,
                                "isUsed": True,
                            },
                            "second_step": {
                                "inputID": 4283122,
                                "toolConfigID": 1024,
                                "taskID": 1896642,
                                "status": "Completed",
                                "outputID": 1947652,
                                "isUsed": True,
                            },
                            "third_step": {
                                "inputID": 1947652,
                                "toolConfigID": 1024,
                                "taskID": 8735432,
                                "status": "Completed",
                                "outputID": 1285429,
                                "isUsed": True,
                            },
                            "Result": {"result": "3489", "isUsed": True},
                        },
                        "rgb(59, 115, 143)",
                    ],
                }
            ]
        }
    else:
        return {
            "points": [
                {
                    "curveNumber": 0,
                    "pointNumber": 145,
                    "pointIndex": 145,
                    "x": 16709,
                    "y": 3703,
                    "bbox": {"x0": 901.12, "x1": 903.12, "y0": 396.842, "y1": 398.842},
                }
            ]
        }


@pytest.fixture()
def metadata(request):
    if request.param == "histogram":
        return {
            "subject": "some_subject",
            "dataset-pipeline": "some_dataset-pipeline",
            "result": "3489",
            "first_step": {
                "status": "Completed",
                "inputID": "6435319",
                "outputID": "4283122",
                "taskID": "5616972",
                "toolConfigID": "678",
            },
            "second_step": {
                "status": "Completed",
                "inputID": "4283122",
                "outputID": "1947652",
                "taskID": "1896642",
                "toolConfigID": "1024",
            },
            "third_step": {
                "status": "Completed",
                "inputID": "1947652",
                "outputID": "1285429",
                "taskID": "8735432",
                "toolConfigID": "1024",
            },
        }
    else:
        return {
            "x": {
                "subject": "sub-2017146_ses-NAPFU48_run-002_T1w.nii.gz",
                "dataset-pipeline": "prevent-AD-FSL",
                "result": "16709",
                "FSL_First": {
                    "status": "Completed",
                    "inputID": "3680817",
                    "outputID": "3681957",
                    "taskID": "1888425",
                    "toolConfigID": "721",
                },
                "Subfolder_File_Extractor_FSL": {
                    "status": "Completed",
                    "inputID": "3681957",
                    "outputID": "3682428",
                    "taskID": "1889025",
                    "toolConfigID": "2094",
                },
                "FSL_Stats": {
                    "status": "Completed",
                    "inputID": "3682428",
                    "outputID": "3682749",
                    "taskID": "1889307",
                    "toolConfigID": "1698",
                },
            },
            "y": {
                "subject": "sub-2017146_ses-NAPFU48_run-002_T1w.nii.gz",
                "dataset-pipeline": "prevent-AD-FreeSurfer",
                "result": "Result: 3703",
                "FreeSurfer_Recon_all": {
                    "status": "Completed",
                    "inputID": "3680817",
                    "outputID": "3683955",
                    "taskID": "1888725",
                    "toolConfigID": "583",
                },
                "Subfolder_File_Extractor_FreeSurfer_1": {
                    "status": "Completed",
                    "inputID": "3683955",
                    "outputID": "3684015",
                    "taskID": "1890132",
                    "toolConfigID": "2094",
                },
                "Subfolder_File_Extractor_FreeSurfer_2": {
                    "status": "Completed",
                    "inputID": "3684015",
                    "outputID": "3684129",
                    "taskID": "1890222",
                    "toolConfigID": "2094",
                },
            },
        }



"""Utility functions and constants of the app."""
import json
import os

import dash_bootstrap_components as dbc
import pandas as pd
import plotly.express as px
from dash import html

COLORS = [
    px.colors.qualitative.G10,
    px.colors.sequential.Teal,
    px.colors.sequential.Brwnyl,
    px.colors.sequential.Burg,
    px.colors.sequential.Purp,
]


def pull_files(path):
    """
    Creates a list of paths for .json files in the input path.

    Parameters
    ----------
    path : str
        Path of the directory containing .json files to be visualized.

    Returns
    -------
    list
        List of paths.
    """
    files = []
    for i in os.listdir(path):
        if i[-5:] == ".json":
            files.append((os.path.join(path, i), i[:-5]))
    return files


def process_file(file, color):
    """
    Creates a dataframe from input file's data.
    Loads the data from the file into a dataframe and adds
    Result, Metadata, and Color (using a single color sequence
    from COLORS per file) columns to the dataframe using
    the data.

    Parameters
    ----------
    file : tuple
        (dataset-pipeline file path, dataset-pipeline name).
    color : int
        Index number for selecting a color sequence from COLORS.

    Returns
    -------
    pandas.core.frame.DataFrame
        A dataframe containing Subject, Dataset-Pipeline, Result
        Metadata, and Color columns.
    """
    data = None
    with open(file[0], "r") as dataset:
        data = json.load(dataset)
    x = []
    for k in data.keys():
        z = len(COLORS[color]) - 1
        for v in data[k].keys():
            x.append(
                (k, v, data[k][v]["Result"]["result"], data[k][v], COLORS[color][z])
            )
            z -= 1
    x = [
        (i[0], i[1], -1, i[3], i[4])
        if i[2] is None
        else (i[0], i[1], float(i[2]), i[3], i[4])
        for i in x
    ]
    df = pd.DataFrame(
        {
            "Subject": [i[0] for i in x],
            "Dataset-Pipeline": [file[1] + "-" + i[1] for i in x],
            "Result": [i[2] for i in x],
            "Metadata": [i[3] for i in x],
            "Color": [i[4] for i in x],
        }
    )
    return df


def process_files(path):
    """
    Creates a dataframe from the data in files located in the
    input path.
    Utilizes pull_files function to apply process_file function
    to multiple files.

    Parameters
    ----------
    path : str
        Path of the directory containing .json files to be visualized.


    Returns
    ----------
    pandas.core.frame.DataFrame
        A dataframe generated through conconcatenating dataframes
        generated by pull_files function for each file.
    """
    files = pull_files(path)
    dfs = []
    for i, j in enumerate(files):
        dfs.append(process_file(j, i))
    return pd.concat(dfs)


def pull_directories(path):
    """
    Generates a list of tuples for each directory located in the input path.
    Tuples are in (directory name, dataframe generated from the files
    in directory) form. Dataframes are generated using pull_files function.

    Parameters
    ----------
    path : str
        Path of the directory containing direcotries that all contain .json files
        to be visualized.

    Returns
    ----------
    list
        A list of tuples in (directory name, directory dataframe) form.
    """
    files = []
    for directory in os.listdir(path):
        files.append((directory, process_files(os.path.join(path, directory))))
    return files


def generate_summary(df):
    """
    Generates a summary for the input dataframe's data.
    The summary includes total number of datapoints, total number
    of missing datapoints, and number of missing datapoints in
    each dataset-pipeline.

    Parameters
    ----------
    df : pandas.core.frame.DataFrame
        Dataframe containing the data.

    Returns
    -------
    dash_bootstrap_components._components.Card.Card
        A structured and styled summary of the dataframe's data.
    """
    total = str(df.shape[0])
    missing = str(df[df["Result"] == -1].shape[0])
    header = html.H4("Summary", className="card-title")
    summary = [
        header,
        "Total number of datapoints: " + total,
        html.Br(),
        "Total number of missing datapoints: " + missing,
        html.Br(),
    ]
    pipelines = df["Dataset-Pipeline"].unique().tolist()
    for p in pipelines:
        s = (
            p
            + ": "
            + str(df[(df["Dataset-Pipeline"] == p) & (df["Result"] == -1)].shape[0])
        )
        summary.append(s)
        summary.append(html.Br())

    return dbc.Card(dbc.CardBody(summary, className="card-text"))


def extract_metadata(clickData, x=None, y=None, df=None):
    """
    Extracts and structures metadata from graphs' click event.
    The extracted metadata is used to process data and create UI elements to
    display the metadata. If x or y is None, it assumes the clickData is coming
    from histogram graph otherwise it assumes its coming from scatter plot graph.

    Parameters
    ----------
    clickData : dict
        Data from latest histogram graph click event.
    x : str, optional
        Name of the x-axis dataset-pipeline.
    y : str, optional
        Name of the y-axis dataset-pipeline.
    df : pandas.core.frame.DataFrame, optional
        Dataframe containing the data.

    Returns
    ----------
    dict
        Structured metadata.
    """
    metadata = {}
    if x is None or y is None:
        metadata["subject"] = clickData["points"][0]["customdata"][0]
        metadata["dataset-pipeline"] = clickData["points"][0]["y"]
        metadata["result"] = (
            "N/A"
            if clickData["points"][0]["x"] == -1
            else str(clickData["points"][0]["x"])
        )
        for k, v in list(clickData["points"][0]["customdata"][2].items())[:-1]:
            metadata[k] = {
                "status": "Incomplete" if v["status"] is None else v["status"],
                "inputID": "N/A" if v["inputID"] is None else str(v["inputID"]),
                "outputID": "N/A" if v["outputID"] is None else str(v["outputID"]),
                "taskID": "N/A" if v["taskID"] is None else str(v["taskID"]),
                "toolConfigID": "N/A"
                if v["toolConfigID"] is None
                else str(v["toolConfigID"]),
            }

    else:
        metadata["x"] = {
            "subject": df[
                (df["Dataset-Pipeline"] == x)
                & (df["Result"] == clickData["points"][0]["x"])
            ]["Subject"].iloc[0],
            "dataset-pipeline": x,
            "result": "N/A"
            if clickData["points"][0]["x"] == -1
            else str(clickData["points"][0]["x"]),
        }
        x_dic = df[
            (df["Dataset-Pipeline"] == x)
            & (df["Result"] == clickData["points"][0]["x"])
        ]["Metadata"].iloc[0]
        for k, v in list(x_dic.items())[:-1]:
            metadata["x"][k] = {
                "status": "Incomplete" if v["status"] is None else v["status"],
                "inputID": "N/A" if v["inputID"] is None else str(v["inputID"]),
                "outputID": "N/A" if v["outputID"] is None else str(v["outputID"]),
                "taskID": "N/A" if v["taskID"] is None else str(v["taskID"]),
                "toolConfigID": "N/A"
                if v["toolConfigID"] is None
                else str(v["toolConfigID"]),
            }
        metadata["y"] = {
            "subject": df[
                (df["Dataset-Pipeline"] == y)
                & (df["Result"] == clickData["points"][0]["y"])
            ]["Subject"].iloc[0],
            "dataset-pipeline": y,
            "result": "N/A"
            if clickData["points"][0]["y"] == -1
            else "Result: " + str(clickData["points"][0]["y"]),
        }
        y_dic = df[
            (df["Dataset-Pipeline"] == y)
            & (df["Result"] == clickData["points"][0]["y"])
        ]["Metadata"].iloc[0]
        for k, v in list(y_dic.items())[:-1]:
            metadata["y"][k] = {
                "status": "Incomplete" if v["status"] is None else v["status"],
                "inputID": "N/A" if v["inputID"] is None else str(v["inputID"]),
                "outputID": "N/A" if v["outputID"] is None else str(v["outputID"]),
                "taskID": "N/A" if v["taskID"] is None else str(v["taskID"]),
                "toolConfigID": "N/A"
                if v["toolConfigID"] is None
                else str(v["toolConfigID"]),
            }

    return metadata



"""
The entry point of the app.
"""
import dash
import dash_bootstrap_components as dbc
from dash import Input, Output, callback

from nerv.callbacks import hist_click_func, plot_scatter_func, scatter_click_func
from nerv.layouts import layout
from nerv.utility import process_files


def start(path, local=True):
    """
    Creates and configures a Dash app instance that can either be
    run locally or used for deployment.

    Parameters
    ----------
    path : str
        Path of the directory containing files to be visualized.
    local : bool, optional
        Whether or not to start the app locally in the terminal or return
        a Flask app containing the Dash app for deployment, by default True.

    Returns
    -------
    None or flask.app.Flask
        None if local parameter is set to True, otherwise
        returns a flask app instance.
    """
    app = dash.Dash(
        __name__,
        routes_pathname_prefix="/",
        external_stylesheets=[dbc.themes.BOOTSTRAP],
    )
    app.title = "NeRV"

    def serve_layout():
        """
        Generates and updates the app layout.
        Used by the app.layout to dynamically generate/update
        app layout on page referesh or whenever change is detected.

        Returns
        -------
        dash_bootstrap_components._components.Container.Container
            The app layout made up of various dash bootstrap components and
            dash core components wrapped with a dash bootstrap componenets
            container component.
        """
        global df
        df = process_files(path)

        return layout(df)

    app.layout = serve_layout

    @callback(Output("hist-metadata-div", "children"), Input("histogram", "clickData"))
    def hist_click(clickData):
        """
        Processes data from the histogram graph click event.

        Parameters
        ----------
        clickData : dict
            Data from latest histogram graph click event.

        Returns
        ----------
        dash._callback.Noupdate or dash_bootstrap_components._components.Card.Card
            No dash update if clickData is None otherwise, styled and structured
            metadata to be displayed in hist-metadata-div.
        """
        return hist_click_func(clickData)

    @callback(
        Output("scatter", "figure"),
        Input("x", "value"),
        Input("y", "value"),
    )
    def plot_scatter(x, y):
        """
        Creates a scatter plot figure.

        Parameters
        ----------
            x : str
                Name of the x-axis dataset-pipeline.
            y : str
                Name of the y-axis dataset-pipeline.

        Returns
        ----------
        plotly.graph_objs._figure.Figure
            A plotly graph figure object for the scatter plot
            where x and y inputs are the axes.
        """
        return plot_scatter_func(x, y, df)

    @callback(
        Output("scatter-metadata-div", "children"),
        Input("scatter", "clickData"),
        Input("x", "value"),
        Input("y", "value"),
    )
    def scatter_click(clickData, x, y):
        """
        Processes data from the scatter plot graph click event.

        Parameters
        ----------
        clickData : dict
            Data from latest histogram graph click event.
        x : str
            Name of the x-axis dataset-pipeline.
        y : str
            Name of the y-axis dataset-pipeline.

        Returns
        ----------
        dash._callback.Noupdate or dash_bootstrap_components._components.Card.Card
            No dash update if clickData is None otherwise, styled and structured
            metadata to be displayed in scatter-metadata-div.
        """
        return scatter_click_func(clickData, x, y, df)

    if local:
        app.run_server()
    else:
        return app.server



"""
Logic of app callback functions.
"""
import dash
import dash_bootstrap_components as dbc
import plotly.express as px
from dash import html

from nerv.utility import extract_metadata


def hist_click_func(clickData):
    """
    Processes data from the histogram graph click event and creates
    UI elements to display the metadata.

    Parameters
    ----------
    clickData : dict
        Data from latest histogram graph click event.

    Returns
    ----------
    dash._callback.Noupdate or dash_bootstrap_components._components.Card.Card
        No dash update if clickData is None otherwise, structured and styled
        metadata to be displayed in hist-metadata-div.
    """
    if not clickData:
        return dash.no_update

    metadata = extract_metadata(clickData)

    card_body_content = [
        html.H4("Metadata", className="card-title"),
        "Subject: " + metadata["subject"],
        html.Br(),
        "Dataset-Pipeline: " + metadata["dataset-pipeline"],
        html.Br(),
        "Result: " + metadata["result"],
        html.Br(),
        "Pipeline steps: ",
        html.Br(),
        html.Br(),
    ]

    for k, v in list(metadata.items())[3:]:
        card_body_content.append(
            dbc.Accordion(
                [
                    dbc.AccordionItem(
                        [
                            "Status: " + v["status"],
                            html.Br(),
                            "Input ID: ",
                            v["inputID"]
                            if v["inputID"] == "N/A"
                            else html.A(
                                str(v["inputID"]),
                                href="https://portal.cbrain.mcgill.ca/userfiles/"
                                + str(v["inputID"]),
                            ),
                            html.Br(),
                            "Output ID: ",
                            v["outputID"]
                            if v["outputID"] == "N/A"
                            else html.A(
                                str(v["outputID"]),
                                href="https://portal.cbrain.mcgill.ca/userfiles/"
                                + str(v["outputID"]),
                            ),
                            html.Br(),
                            "Task ID: ",
                            v["taskID"]
                            if v["taskID"] == "N/A"
                            else html.A(
                                str(v["taskID"]),
                                href="https://portal.cbrain.mcgill.ca/tasks/"
                                + str(v["taskID"]),
                            ),
                            html.Br(),
                            "Tool Configuration ID: " + v["toolConfigID"],
                        ],
                        title=k,
                    )
                ],
                start_collapsed=True,
            )
        )

    return dbc.Card(dbc.CardBody(card_body_content, className="card-text"))


def plot_scatter_func(x, y, df):
    """
    Creates a scatter plot figure.
    If x or y is None, it uses the first and last dataset-pipeline
    in the dataframe as x and y.

    Parameters
    ----------
        x : str
            Name of the x-axis dataset-pipeline.
        y : str
            Name of the y-axis dataset-pipeline.
        df : pandas.core.frame.DataFrame
            Dataframe containing the data.

    Returns
    ----------
    plotly.graph_objs._figure.Figure
        A plotly graph figure object for the scatter plot
        where x and y inputs are the axes.
    """
    if not x or not y:
        return px.scatter(
            df,
            x=df[df["Dataset-Pipeline"] == df["Dataset-Pipeline"].unique().tolist()[0]][
                "Result"
            ],
            y=df[
                df["Dataset-Pipeline"] == df["Dataset-Pipeline"].unique().tolist()[-1]
            ]["Result"],
            marginal_x="histogram",
            marginal_y="histogram",
            color_discrete_sequence=px.colors.qualitative.G10[::-1],
        ).update_layout(
            xaxis={"rangeslider": {"visible": True}},
            xaxis_title=df["Dataset-Pipeline"].unique().tolist()[0],
            yaxis_title=df["Dataset-Pipeline"].unique().tolist()[-1],
        )
    else:
        return px.scatter(
            df,
            x=df[df["Dataset-Pipeline"] == x]["Result"],
            y=df[df["Dataset-Pipeline"] == y]["Result"],
            marginal_x="histogram",
            marginal_y="histogram",
            color_discrete_sequence=px.colors.qualitative.G10[::-1],
        ).update_layout(
            xaxis={"rangeslider": {"visible": True}}, xaxis_title=x, yaxis_title=y
        )


def scatter_click_func(clickData, x, y, df):
    """
    Processes data from the scatter plot graph click event and creates
    UI elements to display the metadata.
    If x or y is None, it uses the first and last dataset-pipeline
    in the dataframe as x and y.

    Parameters
    ----------
    clickData : dict
        Data from latest histogram graph click event.
    x : str
        Name of the x-axis dataset-pipeline.
    y : str
        Name of the y-axis dataset-pipeline.
    df : pandas.core.frame.DataFrame
            Dataframe containing the data.

    Returns
    ----------
    dash._callback.Noupdate or dash_bootstrap_components._components.Card.Card
        No dash update if clickData is None otherwise, structured and styled
        metadata to be displayed in scatter-metadata-div.
    """
    if not clickData:
        return dash.no_update

    elif not x or not y:
        xys = df["Dataset-Pipeline"].unique().tolist()
        x, y = xys[0], xys[-1]

    metadata = extract_metadata(clickData, x, y, df)
    card_body_content = [
        html.H4("Metadata", className="card-title"),
        "Subject: " + metadata["x"]["subject"],
        html.Br(),
        "Dataset-Pipeline: " + metadata["x"]["dataset-pipeline"],
        html.Br(),
        "Result: " + metadata["x"]["result"],
        html.Br(),
        "Pipeline steps: ",
        html.Br(),
        html.Br(),
    ]

    for k, v in list(metadata["x"].items())[3:]:
        card_body_content.append(
            dbc.Accordion(
                [
                    dbc.AccordionItem(
                        [
                            "Status: " + v["status"],
                            html.Br(),
                            "Input ID: ",
                            v["inputID"]
                            if v["inputID"] == "N/A"
                            else html.A(
                                str(v["inputID"]),
                                href="https://portal.cbrain.mcgill.ca/userfiles/"
                                + str(v["inputID"]),
                            ),
                            html.Br(),
                            "Output ID: ",
                            v["outputID"]
                            if v["outputID"] == "N/A"
                            else html.A(
                                str(v["outputID"]),
                                href="https://portal.cbrain.mcgill.ca/userfiles/"
                                + str(v["outputID"]),
                            ),
                            html.Br(),
                            "Task ID: ",
                            v["taskID"]
                            if v["taskID"] == "N/A"
                            else html.A(
                                str(v["taskID"]),
                                href="https://portal.cbrain.mcgill.ca/tasks/"
                                + str(v["taskID"]),
                            ),
                            html.Br(),
                            "Tool Configuration ID: " + v["toolConfigID"],
                        ],
                        title=k,
                    )
                ],
                start_collapsed=True,
            )
        )

    card_body_content += [
        html.Br(),
        "Subject: " + metadata["y"]["subject"],
        html.Br(),
        "Dataset-Pipeline: " + metadata["y"]["dataset-pipeline"],
        html.Br(),
        "Result: " + metadata["y"]["result"],
        html.Br(),
        "Pipeline steps: ",
        html.Br(),
        html.Br(),
    ]
    for k, v in list(metadata["y"].items())[3:]:
        card_body_content.append(
            dbc.Accordion(
                [
                    dbc.AccordionItem(
                        [
                            "Status: " + v["status"],
                            html.Br(),
                            "Input ID: ",
                            v["inputID"]
                            if v["inputID"] == "N/A"
                            else html.A(
                                str(v["inputID"]),
                                href="https://portal.cbrain.mcgill.ca/userfiles/"
                                + str(v["inputID"]),
                            ),
                            html.Br(),
                            "Output ID: ",
                            v["outputID"]
                            if v["outputID"] == "N/A"
                            else html.A(
                                str(v["outputID"]),
                                href="https://portal.cbrain.mcgill.ca/userfiles/"
                                + str(v["outputID"]),
                            ),
                            html.Br(),
                            "Task ID: ",
                            v["taskID"]
                            if v["taskID"] == "N/A"
                            else html.A(
                                str(v["taskID"]),
                                href="https://portal.cbrain.mcgill.ca/tasks/"
                                + str(v["taskID"]),
                            ),
                            html.Br(),
                            "Tool Configuration ID: " + v["toolConfigID"],
                        ],
                        title=k,
                    )
                ],
                start_collapsed=True,
            )
        )

    return dbc.Card(dbc.CardBody(card_body_content, className="card-text"))



"""Layout of the app."""
import dash_bootstrap_components as dbc
import plotly.express as px
from dash import dcc, html

from nerv import utility as util


def layout(df):
    """
    Generates the app layout.

    Parameters
    ----------
    df : pandas.core.frame.DataFrame
        Dataframe containing the data for graphs.

    Returns
    -------
    dash_bootstrap_components._components.Container.Container
        The app layout made up of various dash bootstrap components and
        dash core components wrapped with a dash bootstrap componenets
        container component.
    """
    hist_plot = dcc.Graph(
        id="histogram",
        figure=px.histogram(
            df[df["Result"] != -1],
            x="Result",
            color="Dataset-Pipeline",
            color_discrete_map={
                k: v
                for k, v in zip(
                    df["Dataset-Pipeline"].unique().tolist(),
                    df["Color"].unique().tolist(),
                )
            },
            barmode="overlay",
            marginal="rug",
            hover_data=df.columns,
        )
        .update_layout(
            xaxis_title=r"$\text {Hippocampus Volume } (mm^3)$",
            yaxis_title="Count",
            xaxis={
                "rangeslider": {"visible": True},
                "range": [
                    -1000,
                    df["Result"].max() + 1000,
                ],
            },
        )
        .update_xaxes(
            ticks="outside",
            tickwidth=2,
            tickcolor="white",
            ticklen=10,
        ),
        config={"displaylogo": False},
        mathjax=True,
    )

    summary = util.generate_summary(df)

    hist_metadata = html.Div(id="hist-metadata-div")

    hist_tab = dbc.Tab(
        [
            dbc.Row(
                [
                    dbc.Col(hist_plot, width="9"),
                    dbc.Col(
                        [dbc.Row(dbc.Col(summary)), dbc.Row(dbc.Col(hist_metadata))]
                    ),
                ],
            ),
        ],
        label="Distribution Plot",
    )

    x_dropdown = dcc.Dropdown(
        id="x",
        options=[
            {
                "label": k,
                "value": v,
            }
            for k, v in zip(
                df["Dataset-Pipeline"].unique().tolist(),
                df["Dataset-Pipeline"].unique().tolist(),
            )
        ],
        value=df["Dataset-Pipeline"].unique().tolist()[0],
        placeholder="x",
    )

    y_dropdown = dcc.Dropdown(
        id="y",
        options=[
            {
                "label": k,
                "value": v,
            }
            for k, v in zip(
                df["Dataset-Pipeline"].unique().tolist(),
                df["Dataset-Pipeline"].unique().tolist(),
            )
        ],
        value=df["Dataset-Pipeline"].unique().tolist()[-1],
        placeholder="y",
    )

    scatter_plot = dcc.Graph(
        id="scatter",
        figure=px.scatter(
            df,
            x=df[df["Dataset-Pipeline"] == df["Dataset-Pipeline"].unique().tolist()[0]][
                "Result"
            ],
            y=df[
                df["Dataset-Pipeline"] == df["Dataset-Pipeline"].unique().tolist()[-1]
            ]["Result"],
            marginal_x="histogram",
            marginal_y="histogram",
            color_discrete_sequence=px.colors.qualitative.G10[::-1],
        ).update_layout(
            xaxis={"rangeslider": {"visible": True}},
            xaxis_title=df["Dataset-Pipeline"].unique().tolist()[0],
            yaxis_title=df["Dataset-Pipeline"].unique().tolist()[-1],
        ),
        config={"displaylogo": False},
    )

    scatter_metadata = html.Div(id="scatter-metadata-div")

    scatter_tab = dbc.Tab(
        [
            dbc.Row(
                [
                    dbc.Col(
                        [
                            dbc.Row([dbc.Col(x_dropdown), dbc.Col(y_dropdown)]),
                            dbc.Row(dbc.Col(scatter_plot)),
                        ],
                        width="9",
                    ),
                    dbc.Col(scatter_metadata),
                ]
            )
        ],
        label="Joint Plot",
    )

    layout = dbc.Container(
        [
            dbc.Tabs(
                [hist_tab, scatter_tab],
            ),
        ],
        fluid=True,
    )

    return layout



import codecs
import os

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as file:
    long_description = "\n" + file.read()

with open("requirements.txt", "r") as file:
    install_requires = file.read().split("\n")

VERSION = "0.1.1"
DESCRIPTION = "Neuroimaging Results Visualization"
LONG_DESCRIPTION = long_description

setup(
    name="nerv",
    version=VERSION,
    author="rmanaem",
    author_email="<rmanaem@protonmail.ch>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="https://github.com/rmanaem/nerv",
    packages=find_packages(),
    include_package_data=True,
    package_data={"nerv": ["assets/*"]},
    install_requires=install_requires,
    license="MIT",
    keywords=[
        "python",
        "dash",
        "pandas",
        "data visualization",
        "plotly",
        "neuroscience",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "License :: OSI Approved :: MIT License",
    ],
)
